# == Class: smf::service
#
# This class provides a convenience function to make sure demons get
# integrated with Solaris SMF frameworks.
# Additionally, it will update the SMF configuration, if the config is
# changed on the puppet side.
#
# Note that this is a defined type. As such, the syntax to use is slightly
# different from a regular class
#
# === Parameters
#
#
# [*name*]
#   Set the name of the service, as the name of the resource
#
# [*service_command*]
#   Specify the command-line that should be run to start the service.
#
# [*service_conf*]
#   Optional argument, to define a config file for the demon.
#   If the file is changed, then the demon will automatically be 'refreshed',
#   with a HUP signal, presuming you have enabled that option also
#
# [*hup_refresh*]
#   Optional argument, that enables use of 'svcadm refresh' on your service.
#
# [*smftree*]
#   You probably dont want to change this. Defaults to "site", for
#   svc://site/$name
#
#
# === Examples
#
#  smf::service { "somename": service_command => "/usr/sbin/somename"}
#
#  (note the lack of "class" keyword)
#
# === Notes
#
# We make the assumption it is a "network" dependant service,
# so we add a dependancy on /network/service in the SMF XML soup.
#
# To keep ease of use, we use a generic common 'init' style template
# for start/stop method, as well as a common SMF XML definition template
# If you want to get fancier, you should probably create your own
# separate smf::yourservice class, with associated XML and startup script.
#
# === Authors
#
# Philip Brown <phil@bolthole.com>
#
# === Copyright
#
# Copyright 2014 Philip Brown
#
# Freely usable and redistributable with credit given to author
#

define smf::service (
	$service_command,
	$service_conf = undef,
	$hup_refresh = undef,
	$smftree = "site",
 )
{

	$servicename="$name"

	$xmltmp="/var/run/smf-${servicename}.xml"
	$xmldelivery="/var/svc/manifest/$smftree/$servicename.xml"
	$xmlcontent=template("smf/generic.xml")

	$svcdelivery="/lib/svc/method/svc-${servicename}"

	# SMF definition
	file {
		"$xmldelivery":
		ensure => present,
		owner => root,
		group => sys,
		content=> $xmlcontent,
		# 'service_command' gets used in the template
		notify => Exec["smf-$servicename-configure"],
	}


	# Start/Stop scripts
	file { "$svcdelivery":
		ensure => present,
		owner => root,
		group => bin,
		mode => '0555',
		content=> template("smf/svc-generic"),
		notify => Exec["smf-$servicename-configure"],
	}

	###############################################################
	# Below here, One-time hooks to configure SMF to know about the service

	# One-time tmp destination.
	file { "${xmltmp}":
		owner => root,
		group => sys,
		content => $xmlcontent,
		# Yes having this in two locations is a bit wierd.
		# but it seems to be what is required
		# svccfg sometimes complains if we import from /var/svc
		require     => File["$svcdelivery"],
	}

	# Defaults to starting service, on configure.
	exec { "smf-$servicename-configure":
		command     => "/usr/sbin/svccfg import $xmltmp",
		refreshonly => true,
		require     => File["$xmltmp"],
	}



}

